CaveDirt = {}
CaveDirt[1] = { "desert_cave_ground", 0 }
CaveDirt[2] = { "desert_cave_ground", 1 }
CaveDirt[3] = { "desert_cave_ground", 2 }

CaveDirt_CaveFloor = {}
CaveDirt_CaveFloor[1]  = { "desert_cave_ground", 3 }  -- northwest border
CaveDirt_CaveFloor[2]  = { "desert_cave_ground", 39 } -- northern border
CaveDirt_CaveFloor[3]  = { "desert_cave_ground", 5 }  -- northeast border
CaveDirt_CaveFloor[4]  = { "desert_cave_ground", 22 } -- eastern border
CaveDirt_CaveFloor[5]  = { "desert_cave_ground", 37 } -- southeastern border
CaveDirt_CaveFloor[6]  = { "desert_cave_ground", 7 }  -- southern border
CaveDirt_CaveFloor[7]  = { "desert_cave_ground", 35 } -- southwestern border
CaveDirt_CaveFloor[8]  = { "desert_cave_ground", 24 } -- western border
CaveDirt_CaveFloor[9]  = { "desert_cave_ground", 40 } -- northwestern corner
CaveDirt_CaveFloor[10] = { "desert_cave_ground", 38 } -- northeastern corner
CaveDirt_CaveFloor[11] = { "desert_cave_ground", 6 }  -- southeastern corner
CaveDirt_CaveFloor[12] = { "desert_cave_ground", 8 }  -- southwestern corner

CaveFloor = {}
CaveFloor[1]  = { "desert_cave_ground", 9 }
CaveFloor[2]  = { "desert_cave_ground", 10 }
CaveFloor[3]  = { "desert_cave_ground", 11 }
CaveFloor[4]  = { "desert_cave_ground", 12 }
CaveFloor[5]  = { "desert_cave_ground", 25 }
CaveFloor[6]  = { "desert_cave_ground", 26 }
CaveFloor[7]  = { "desert_cave_ground", 27 }
CaveFloor[8]  = { "desert_cave_ground", 28 }
CaveFloor[9]  = { "desert_cave_ground", 41 }
CaveFloor[10] = { "desert_cave_ground", 42 }
CaveFloor[11] = { "desert_cave_ground", 43 }
CaveFloor[12] = { "desert_cave_ground", 44 }

CaveFloor_CaveDirt = {}
CaveFloor_CaveDirt[1]  = { "desert_cave_ground", 6 }  -- northwest border
CaveFloor_CaveDirt[2]  = { "desert_cave_ground", 7 }  -- northern border
CaveFloor_CaveDirt[3]  = { "desert_cave_ground", 8 }  -- northeast border
CaveFloor_CaveDirt[4]  = { "desert_cave_ground", 24 } -- eastern border
CaveFloor_CaveDirt[5]  = { "desert_cave_ground", 40 } -- southeastern border
CaveFloor_CaveDirt[6]  = { "desert_cave_ground", 39 } -- southern border
CaveFloor_CaveDirt[7]  = { "desert_cave_ground", 38 } -- southwestern border
CaveFloor_CaveDirt[8]  = { "desert_cave_ground", 22 } -- western border
CaveFloor_CaveDirt[9]  = { "desert_cave_ground", 37 } -- northwestern corner
CaveFloor_CaveDirt[10] = { "desert_cave_ground", 35 } -- northeastern corner
CaveFloor_CaveDirt[11] = { "desert_cave_ground", 3 }  -- southeastern corner
CaveFloor_CaveDirt[12] = { "desert_cave_ground", 5 }  -- southwestern corner

CaveFloor_CaveDirt_East = {}
CaveFloor_CaveDirt_East[1] = { "desert_cave_ground", 19 }
CaveFloor_CaveDirt_East[2] = { "desert_cave_ground", 24 }

CaveFloor_CaveDirt_North = {}
CaveFloor_CaveDirt_North[1] = { "desert_cave_ground", 7 }
CaveFloor_CaveDirt_North[2] = { "desert_cave_ground", 36 }

CaveFloor_CaveDirt_South = {}
CaveFloor_CaveDirt_South[1] = { "desert_cave_ground", 4 }
CaveFloor_CaveDirt_South[2] = { "desert_cave_ground", 39 }

CaveFloor_CaveDirt_West = {}
CaveFloor_CaveDirt_West[1] = { "desert_cave_ground", 21 }
CaveFloor_CaveDirt_West[2] = { "desert_cave_ground", 22 }

TameSand = {}
TameSand[1]  = { "desert_ground", 0 }
TameSand[2]  = { "desert_ground", 1 }
TameSand[3]  = { "desert_ground", 2 }
TameSand[4]  = { "desert_ground", 3 }
TameSand[5]  = { "desert_ground", 8 }
TameSand[6]  = { "desert_ground", 16 }
TameSand[7]  = { "desert_ground", 17 }
TameSand[8]  = { "desert_ground", 18 }
TameSand[9]  = { "desert_ground", 19 }
TameSand[10] = { "desert_ground", 24 }
TameSand[11] = { "desert_ground", 32 }
TameSand[12] = { "desert_ground", 33 }
TameSand[13] = { "desert_ground", 34 }
TameSand[14] = { "desert_ground", 35 }
TameSand[15] = { "desert_ground", 40 }
TameSand[16] = { "desert_ground", 48 }
TameSand[17] = { "desert_ground", 49 }
TameSand[18] = { "desert_ground", 50 }
TameSand[19] = { "desert_ground", 51 }

TameSand_TameSandstones = {}
TameSand_TameSandstones[1]  = { "desert_ground", 4 }  -- northwest border
TameSand_TameSandstones[2]  = { "desert_ground", 38 } -- northern border
TameSand_TameSandstones[3]  = { "desert_ground", 36 } -- northeast border
TameSand_TameSandstones[4]  = { "desert_ground", 21 } -- eastern border
TameSand_TameSandstones[5]  = { "desert_ground", 52 } -- southeastern border
TameSand_TameSandstones[6]  = { "desert_ground", 6 }  -- southern border
TameSand_TameSandstones[7]  = { "desert_ground", 20 } -- southwestern border
TameSand_TameSandstones[8]  = { "desert_ground", 23 } -- western border
TameSand_TameSandstones[9]  = { "desert_ground", 39 } -- northwestern corner
TameSand_TameSandstones[10] = { "desert_ground", 37 } -- northeastern corner
TameSand_TameSandstones[11] = { "desert_ground", 5 }  -- southeastern corner
TameSand_TameSandstones[12] = { "desert_ground", 7 }  -- southwestern corner

TameSandstones = {}
TameSandstones[1] = { "desert_ground", 22 }
TameSandstones[2] = { "desert_ground", 53 }
TameSandstones[3] = { "desert_ground", 54 }
TameSandstones[4] = { "desert_ground", 55 }
TameSandstones[5] = { "desert_ground", 56 }

TameSandstones_TameSand = {}
TameSandstones_TameSand[1]  = { "desert_ground", 5 }  -- northwest border
TameSandstones_TameSand[2]  = { "desert_ground", 6 }  -- northern border
TameSandstones_TameSand[3]  = { "desert_ground", 7 }  -- northeast border
TameSandstones_TameSand[4]  = { "desert_ground", 23 } -- eastern border
TameSandstones_TameSand[5]  = { "desert_ground", 39 } -- southeastern border
TameSandstones_TameSand[6]  = { "desert_ground", 38 } -- southern border
TameSandstones_TameSand[7]  = { "desert_ground", 37 } -- southwestern border
TameSandstones_TameSand[8]  = { "desert_ground", 21 } -- western border
TameSandstones_TameSand[9]  = { "desert_ground", 52 } -- northwestern corner
TameSandstones_TameSand[10] = { "desert_ground", 20 } -- northeastern corner
TameSandstones_TameSand[11] = { "desert_ground", 4 }  -- southeastern corner
TameSandstones_TameSand[12] = { "desert_ground", 36 } -- southwestern corner

WildSand = {}
WildSand[1] = { "desert_ground", 9 }
WildSand[2] = { "desert_ground", 41 }
WildSand[3] = { "desert_ground", 45 }
WildSand[4] = { "desert_ground", 58 }
WildSand[5] = { "desert_ground", 59 }
WildSand[6] = { "desert_ground", 60 }
WildSand[7] = { "desert_ground", 123 }

WildWeeds = {}
WildWeeds[1] = { "desert_ground", 13 }
WildWeeds[2] = { "desert_ground", 14 }
WildWeeds[3] = { "desert_ground", 15 }
WildWeeds[4] = { "desert_ground", 25 }
WildWeeds[5] = { "desert_ground", 29 }
WildWeeds[6] = { "desert_ground", 30 }
WildWeeds[7] = { "desert_ground", 31 }
WildWeeds[8] = { "desert_ground", 57 }

YellowPebbles = {}
YellowPebbles[1] = { "desert_ground", 87 }
YellowPebbles[2] = { "desert_ground", 118 }
YellowPebbles[3] = { "desert_ground", 119 }
YellowPebbles[4] = { "desert_ground", 120 }

YellowPebbles_YellowSand = {}
YellowPebbles_YellowSand[1]  = { "desert_ground", 70 }  -- northwest border
YellowPebbles_YellowSand[2]  = { "desert_ground", 71 }  -- northern border
YellowPebbles_YellowSand[3]  = { "desert_ground", 72 }  -- northeast border
YellowPebbles_YellowSand[4]  = { "desert_ground", 88 }  -- eastern border
YellowPebbles_YellowSand[5]  = { "desert_ground", 104 } -- southeastern border
YellowPebbles_YellowSand[6]  = { "desert_ground", 103 } -- southern border
YellowPebbles_YellowSand[7]  = { "desert_ground", 102 } -- southwestern border
YellowPebbles_YellowSand[8]  = { "desert_ground", 86 }  -- western border
YellowPebbles_YellowSand[9]  = { "desert_ground", 100 } -- northwestern corner
YellowPebbles_YellowSand[10] = { "desert_ground", 101 } -- northeastern corner
YellowPebbles_YellowSand[11] = { "desert_ground", 117 } -- southeastern corner
YellowPebbles_YellowSand[12] = { "desert_ground", 116 } -- southwestern corner

YellowSand = {}
YellowSand[1] = { "desert_ground", 68 }
YellowSand[2] = { "desert_ground", 69 }
YellowSand[3] = { "desert_ground", 84 }
YellowSand[4] = { "desert_ground", 85 }

YellowSand_YellowPebbles = {}
YellowSand_YellowPebbles[1]  = { "desert_ground", 117 } -- northwest border
YellowSand_YellowPebbles[2]  = { "desert_ground", 103 } -- northern border
YellowSand_YellowPebbles[3]  = { "desert_ground", 116 } -- northeast border
YellowSand_YellowPebbles[4]  = { "desert_ground", 86 }  -- eastern border
YellowSand_YellowPebbles[5]  = { "desert_ground", 100 } -- southeastern border
YellowSand_YellowPebbles[6]  = { "desert_ground", 71 }  -- southern border
YellowSand_YellowPebbles[7]  = { "desert_ground", 101 } -- southwestern border
YellowSand_YellowPebbles[8]  = { "desert_ground", 88 }  -- western border
YellowSand_YellowPebbles[9]  = { "desert_ground", 104 } -- northwestern corner
YellowSand_YellowPebbles[10] = { "desert_ground", 102 } -- northeastern corner
YellowSand_YellowPebbles[11] = { "desert_ground", 70 }  -- southeastern corner
YellowSand_YellowPebbles[12] = { "desert_ground", 72 }  -- southwestern corner

CrackedEarth = {}
CrackedEarth[1] = { "mountain_landscape", 10 }
CrackedEarth[2] = { "mountain_landscape", 26 }
CrackedEarth[3] = { "mountain_landscape", 42 }

CrackedEarth_Grass = {}
CrackedEarth_Grass[1]  = { "mountain_landscape", 11 }  -- northwest border
CrackedEarth_Grass[2]  = { "mountain_landscape", 12 }  -- northern border
CrackedEarth_Grass[3]  = { "mountain_landscape", 13 }  -- northeast border
CrackedEarth_Grass[4]  = { "mountain_landscape", 29 }  -- eastern border
CrackedEarth_Grass[5]  = { "mountain_landscape", 45 }  -- southeastern border
CrackedEarth_Grass[6]  = { "mountain_landscape", 44 }  -- southern border
CrackedEarth_Grass[7]  = { "mountain_landscape", 43 }  -- southwestern border
CrackedEarth_Grass[8]  = { "mountain_landscape", 27 }  -- western border
CrackedEarth_Grass[9]  = { "mountain_landscape", 110 } -- northwestern corner
CrackedEarth_Grass[10] = { "mountain_landscape", 111 } -- northeastern corner
CrackedEarth_Grass[11] = { "mountain_landscape", 127 } -- southeastern corner
CrackedEarth_Grass[12] = { "mountain_landscape", 126 } -- southwestern corner

CrackedEarth_Pavers = {}
CrackedEarth_Pavers[1]  = { "mountain_landscape", 63 }  -- northwest border
CrackedEarth_Pavers[2]  = { "mountain_landscape", 92 }  -- northern border
CrackedEarth_Pavers[3]  = { "mountain_landscape", 62 }  -- northeast border
CrackedEarth_Pavers[4]  = { "mountain_landscape", 75 }  -- eastern border
CrackedEarth_Pavers[5]  = { "mountain_landscape", 46 }  -- southeastern border
CrackedEarth_Pavers[6]  = { "mountain_landscape", 60 }  -- southern border
CrackedEarth_Pavers[7]  = { "mountain_landscape", 47 }  -- southwestern border
CrackedEarth_Pavers[8]  = { "mountain_landscape", 77 }  -- western border
CrackedEarth_Pavers[9]  = { "mountain_landscape", 93 }  -- northwestern corner
CrackedEarth_Pavers[10] = { "mountain_landscape", 91 }  -- northeastern corner
CrackedEarth_Pavers[11] = { "mountain_landscape", 59 }  -- southeastern corner
CrackedEarth_Pavers[12] = { "mountain_landscape", 61 }  -- southwestern corner

Grass = {}
Grass[1] = { "mountain_landscape", 78 }
Grass[2] = { "mountain_landscape", 79 }
Grass[3] = { "mountain_landscape", 94 }
Grass[4] = { "mountain_landscape", 95 }

Grass_CrackedEarth = {}
Grass_CrackedEarth[1]  = { "mountain_landscape", 127 } -- northwest border
Grass_CrackedEarth[2]  = { "mountain_landscape", 44 }  -- northern border
Grass_CrackedEarth[3]  = { "mountain_landscape", 126 } -- northeast border
Grass_CrackedEarth[4]  = { "mountain_landscape", 27 }  -- eastern border
Grass_CrackedEarth[5]  = { "mountain_landscape", 110 } -- southeastern border
Grass_CrackedEarth[6]  = { "mountain_landscape", 12 }  -- southern border
Grass_CrackedEarth[7]  = { "mountain_landscape", 111 } -- southwestern border
Grass_CrackedEarth[8]  = { "mountain_landscape", 29 }  -- western border
Grass_CrackedEarth[9]  = { "mountain_landscape", 45 }  -- northwestern corner
Grass_CrackedEarth[10] = { "mountain_landscape", 43 }  -- northeastern corner
Grass_CrackedEarth[11] = { "mountain_landscape", 11 }  -- southeastern corner
Grass_CrackedEarth[12] = { "mountain_landscape", 13 }  -- southwestern corner

Pavers = {}
Pavers[1] = { "mountain_landscape", 14 }
Pavers[2] = { "mountain_landscape", 15 }
Pavers[3] = { "mountain_landscape", 30 }
Pavers[4] = { "mountain_landscape", 31 }
Pavers[5] = { "mountain_landscape", 76 }

Pavers_CrackedEarth = {}
Pavers_CrackedEarth[1]  = { "mountain_landscape", 59 }  -- northwest border
Pavers_CrackedEarth[2]  = { "mountain_landscape", 60 }  -- northern border
Pavers_CrackedEarth[3]  = { "mountain_landscape", 61 }  -- northeast border
Pavers_CrackedEarth[4]  = { "mountain_landscape", 77 }  -- eastern border
Pavers_CrackedEarth[5]  = { "mountain_landscape", 93 }  -- southeastern border
Pavers_CrackedEarth[6]  = { "mountain_landscape", 92 }  -- southern border
Pavers_CrackedEarth[7]  = { "mountain_landscape", 91 }  -- southwestern border
Pavers_CrackedEarth[8]  = { "mountain_landscape", 75 }  -- western border
Pavers_CrackedEarth[9]  = { "mountain_landscape", 46 }  -- northwestern corner
Pavers_CrackedEarth[10] = { "mountain_landscape", 47 }  -- northeastern corner
Pavers_CrackedEarth[11] = { "mountain_landscape", 63 }  -- southeastern corner
Pavers_CrackedEarth[12] = { "mountain_landscape", 62 }  -- southwestern corner

