-- default settings file, use to restore default values when needed
settings = {}
settings.first_start = 1
settings.language = "en@quot"

settings.video_settings = {}
settings.video_settings.full_screen = false
settings.video_settings.screen_resx = 800
settings.video_settings.screen_resy = 600
settings.video_settings.smooth_graphics = true
settings.video_settings.ui_theme = "Royal Silk"

settings.audio_settings = {}
settings.audio_settings.music_vol = 0.7
settings.audio_settings.sound_vol = 0.8

settings.key_settings = {}
settings.key_settings.up = 1073741906
settings.key_settings.down = 1073741905
settings.key_settings.left = 1073741904
settings.key_settings.right = 1073741903
settings.key_settings.confirm = 102
settings.key_settings.cancel = 100
settings.key_settings.menu = 115
settings.key_settings.minimap = 99
settings.key_settings.pause = 32

settings.joystick_settings = {}
settings.joystick_settings.input_disabled = false
settings.joystick_settings.x_axis = 0
settings.joystick_settings.y_axis = 1
settings.joystick_settings.threshold = 8192
settings.joystick_settings.confirm = 0
settings.joystick_settings.cancel = 1
settings.joystick_settings.menu = 2
settings.joystick_settings.minimap = 3
settings.joystick_settings.pause = 6
settings.joystick_settings.quit = 7

settings.game_options = {}
settings.game_options.message_speed = 30 -- characters by second

-- Not supported yet
settings.joystick_settings.index = 0
